@@ -1,10 +1,10 @@
 package Net::DNS::Packet;
 
 #
-# $Id: Packet.pm 979 2012-01-27 08:43:38Z willem $
+# $Id: Packet.pm 1023 2012-10-05 13:58:51Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 979 $)[1];
+$VERSION = (qw$LastChangedRevision: 1023 $)[1];
 
 
 =head1 NAME
@@ -15,9 +15,9 @@ Net::DNS::Packet - DNS protocol packet
 
     use Net::DNS::Packet;
 
-    $packet = new Net::DNS::Packet('example.com', 'MX', 'IN');
+    $query = new Net::DNS::Packet('example.com', 'MX', 'IN');
 
-    $resolver->send($packet);
+    $reply = $resolver->send($query);
 
 
 =head1 DESCRIPTION
@@ -30,17 +30,16 @@ A C<Net::DNS::Packet> object represents a DNS protocol packet.
 use strict;
 use integer;
 use Carp;
-use Net::DNS;
+
+use Exporter 'import';
+use vars qw(@EXPORT_OK);
+@EXPORT_OK = qw(dn_expand);
+
+use Net::DNS::Header;
 use Net::DNS::Question;
 use Net::DNS::RR;
 
 
-use vars qw(@ISA @EXPORT_OK);
-require Exporter;
-@ISA = qw(Exporter);
-@EXPORT_OK = qw(dn_expand);
-
-
 =head1 METHODS
 
 =head2 new
@@ -57,23 +56,6 @@ to A and IN.
 
 If called with an empty argument list, C<new> creates an empty packet.
 
-
-    $packet = new Net::DNS::Packet(\$data);
-    $packet = new Net::DNS::Packet(\$data, 1);		# set debugging
-
-    ($packet, $err) = new Net::DNS::Packet(\$data);
-
-If passed a reference to a scalar containing DNS packet data,
-C<new> creates a packet object by decoding the data.  The optional
-second argument can be passed to turn on debugging output.
-
-If called in array context, returns a packet object and an
-error string.  The content of the error string is unspecified
-if the packet object was successfully created.
-
-Returns undef if unable to create a packet object (e.g., if
-the packet data is truncated).
-
 =cut
 
 sub new {
@@ -81,7 +63,6 @@ sub new {
 	my $class = shift;
 
 	my $self = bless {
-		header	   => Net::DNS::Header->new,
 		question   => [],
 		answer	   => [],
 		authority  => [],
@@ -89,78 +70,119 @@ sub new {
 
 	$self->{question} = [Net::DNS::Question->new(@_)] if @_;
 
+	$self->header->rd(1);
 	return $self;
 }
 
 
+#=head2 decode
+
+=pod
+
+    $packet = new Net::DNS::Packet(\$data);
+    $packet = new Net::DNS::Packet(\$data, 1);		# set debugging
+
+If passed a reference to a scalar containing DNS packet data,
+a new packet object is created by decoding the data.  The optional
+second boolean argument is used to enable debugging output.
+
+Returns undef if unable to create a packet object.
+
+Decoding errors, including data corruption and truncation,
+are collected in the $@ ($EVAL_ERROR) variable.
+
+
+    ($packet, $length) = new Net::DNS::Packet(\$data);
+
+If called in array context, returns a packet object and the
+number of octets successfully decoded.
+
+=cut
+
+use constant HEADER_LENGTH => length pack 'n6', (0) x 6;
+
 sub decode {
 	my $class = shift;
 	my $data  = shift;
 	my $debug = shift || 0;
 
-	my $self = eval {
-		my %self = (
+	my $offset = 0;
+	my $self;
+	eval {
+		die 'corrupt wire-format data' if length($$data) < HEADER_LENGTH;
+
+		$self = bless {
 			question   => [],
 			answer	   => [],
 			authority  => [],
 			additional => [],
-			answersize => length $$data,
-			buffer	   => $data
-			);
+			answersize => length $$data
+			}, $class;
 
-		# Parse header section
-		my ( $header, $offset ) = decode Net::DNS::Header($data);
-		$self{header} = $header;
+		# header section
+		my $header = $self->header;
+		$header->decode($data);
+		my ( $qd, $an, $ns, $ar ) = map { $header->$_ } qw(qdcount ancount nscount arcount);
+		$offset = HEADER_LENGTH;
 
-		# Parse question/zone section
-		for ( 1 .. $header->qdcount ) {
-			my $qd;
-			( $qd, $offset ) = decode Net::DNS::Question( $data, $offset );
-			CORE::push( @{$self{question}}, $qd );
+		# question/zone section
+		my $hash = {};
+		my $record;
+		while ( $qd-- ) {
+			( $record, $offset ) = decode Net::DNS::Question( $data, $offset, $hash );
+			CORE::push( @{$self}{question}, $record );
 		}
 
-		# Retain offset for on-demand decoding of remaining data
-		$self{offset} = $offset;
+		# RR sections
+		while ( $an-- ) {
+			( $record, $offset ) = decode Net::DNS::RR( $data, $offset, $hash );
+			CORE::push( @{$self}{answer}, $record );
+		}
 
-		bless \%self, $class;
+		while ( $ns-- ) {
+			( $record, $offset ) = decode Net::DNS::RR( $data, $offset, $hash );
+			CORE::push( @{$self}{authority}, $record );
+		}
+
+		while ( $ar-- ) {
+			( $record, $offset ) = decode Net::DNS::RR( $data, $offset, $hash );
+			CORE::push( @{$self}{additional}, $record );
+		}
 	};
 
-	( $self || die $@ )->print if $debug;
+	if ( $debug && $self ) {
+		local $@;
+		$self->print;
+	}
 
-	return wantarray ? ( $self, $@ ) : $self;
+	return wantarray ? ( $self, $offset ) : $self;
 }
 
 
 =head2 data
 
     $data = $packet->data;
+    $data = $packet->data($limit);
 
 Returns the packet data in binary format, suitable for sending to
 a nameserver.
 
 =cut
 
+
 sub encode {&data}
 
 sub data {
 	my $self = shift;
 
-	return ${$self->{buffer}} if $self->{buffer};		# retransmit raw packet
+	for ( my $edns = $self->edns ) {			# EDNS support
+		my @xopt = grep { $_->type ne 'OPT' } @{$self->{additional}};
+		$self->{additional} = $edns->default ? [@xopt] : [$edns, @xopt];
+	}
 
-	#----------------------------------------------------------------------
-	# Set record counts in packet header
-	#----------------------------------------------------------------------
-	my $header = $self->{header};
-	$header->qdcount( scalar @{$self->{question}} );
-	$header->ancount( scalar @{$self->{answer}} );
-	$header->nscount( scalar @{$self->{authority}} );
-	$header->arcount( scalar @{$self->{additional}} );
+	my $data = $self->header->encode;			# packet header
 
-	#----------------------------------------------------------------------
-	# Get the data for each section in the packet
-	#----------------------------------------------------------------------
-	my $data = $header->encode;
-	my $hash = {};
+	my $hash = {};						# packet body
 	foreach my $component ( @{$self->{question}},
 				@{$self->{answer}},
 				@{$self->{authority}},
@@ -176,13 +198,64 @@ sub data {
 
     $header = $packet->header;
 
-Returns a C<Net::DNS::Header> object representing the header section
+Constructor method which returns a reference to the unique
+C<Net::DNS::Header> object which represents the header section
 of the packet.
 
 =cut
 
 sub header {
-	return shift->{header};
+	my $self = shift;
+	$self->{header} ||= new Net::DNS::Header($self);
+}
+
+
+=head2 EDNS extended header
+
+    $edns    = $packet->edns;
+    $version = $edns->version;
+    $size    = $edns->size;
+
+Auxilliary function edns() provides access to EDNS extensions.
+
+=cut
+
+sub edns {
+	shift->header->edns;
+}
+
+
+=head2 reply
+
+    $reply = $query->reply( $UDPmax );
+
+Constructor method which returns a new reply packet.
+
+The optional UDPsize argument is the maximum UDP packet size which
+can be reassembled by the local network stack, and is advertised in
+response to an EDNS query.
+
+=cut
+
+sub reply {
+	my $query  = shift;
+	my $UDPmax = shift;
+	die 'erroneous qr flag in query packet' if $query->header->qr;
+
+	my $reply  = new Net::DNS::Packet();
+	my $header = $reply->header;
+	$header->qr(1);						# reply with same id, opcode and question
+	$header->id( $query->header->id );
+	$header->opcode( $query->header->opcode );
+	$reply->{question} = [$query->question];
+
+	$header->rcode('FORMERR');				# failure to provide RCODE is sinful!
+
+	$header->rd( $query->header->rd );			# copy these flags into reply
+	$header->cd( $query->header->cd );
+
+	$reply->edns->size($UDPmax) unless $query->edns->default;
+	return $reply;
 }
 
 
@@ -219,38 +292,12 @@ must not preexist.
 =cut
 
 sub answer {
-	my ($self) = @_;
-	my $rrlist = $self->{answer};
-	return @$rrlist if @$rrlist;
-	@$rrlist = $self->_section( $self->{header}->ancount );
+	return @{shift->{answer}};
 }
 
 sub pre		 {&answer}
 sub prerequisite {&answer}
 
-sub _section {
-	my $self = shift;
-	my $count = shift || return ();
-
-	my $offset = $self->{offset} || return ();
-	my $data   = $self->{buffer} || return ();
-	my $hash   = {};
-	my $byte   = $offset;
-	my @rr;
-	eval {
-		my $rr;
-		undef $self->{offset};
-		while ( $count-- ) {
-			$byte = $offset;
-			( $rr, $offset ) = decode Net::DNS::RR( $data, $offset, $hash );
-			CORE::push( @rr, $rr );
-		}
-		$self->{offset} = $offset;
-	};
-	carp "$@ RR at octet $byte corrupt/incomplete" if $@;
-	return @rr;
-}
-
 
 =head2 authority, update
 
@@ -265,11 +312,7 @@ specifies the RRs or RRsets to be added or deleted.
 =cut
 
 sub authority {
-	my ($self) = @_;
-	my $rrlist = $self->{authority};
-	return @$rrlist if @$rrlist;
-	&answer;
-	@$rrlist = $self->_section( $self->{header}->nscount );
+	return @{shift->{authority}};
 }
 
 sub update {&authority}
@@ -285,13 +328,7 @@ section of the packet.
 =cut
 
 sub additional {
-	my ($self) = @_;
-	my $rrlist = $self->{additional};
-	return @$rrlist if @$rrlist;
-	&authority;
-	@$rrlist = $self->_section( $self->{header}->arcount );
-	undef $self->{buffer};
-	return @$rrlist;
+	return @{shift->{additional}};
 }
 
 
@@ -318,38 +355,38 @@ Returns a string representation of the packet.
 sub string {
 	my $self = shift;
 
-	my $header = $self->{header};
+	my $header = $self->header;
 	my $update = $header->opcode eq 'UPDATE';
 
 	my $server = $self->{answerfrom};
 	my $string = $server ? ";; Answer received from $server ($self->{answersize} bytes)\n" : "";
 
-	$string .= ";; HEADER SECTION\n".$header->string;
+	$string .= ";; HEADER SECTION\n" . $header->string;
 
 	my $question = $update ? 'ZONE' : 'QUESTION';
 	my @question = map $_->string, $self->question;
-	my $qdcount = @question;
-	my $qds = $qdcount != 1 ? 's' : '';
+	my $qdcount  = scalar @question;
+	my $qds	     = $qdcount != 1 ? 's' : '';
 	$string .= join "\n;; ", "\n;; $question SECTION ($qdcount record$qds)", @question;
 
 	my $answer = $update ? 'PREREQUISITE' : 'ANSWER';
-	my @answer = map $_->string, $self->answer;
-	my $ancount = @answer;
-	my $ans = $ancount != 1 ? 's' : '';
+	my @answer  = map $_->string, $self->answer;
+	my $ancount = scalar @answer;
+	my $ans	    = $ancount != 1 ? 's' : '';
 	$string .= join "\n", "\n\n;; $answer SECTION ($ancount record$ans)", @answer;
 
 	my $authority = $update ? 'UPDATE' : 'AUTHORITY';
 	my @authority = map $_->string, $self->authority;
-	my $nscount = @authority;
-	my $nss = $nscount != 1 ? 's' : '';
+	my $nscount   = scalar @authority;
+	my $nss	      = $nscount != 1 ? 's' : '';
 	$string .= join "\n", "\n\n;; $authority SECTION ($nscount record$nss)", @authority;
 
 	my @additional = map $_->string, $self->additional;
-	my $arcount = @additional;
-	my $ars = $arcount != 1 ? 's' : '';
+	my $arcount    = scalar @additional;
+	my $ars	       = $arcount != 1 ? 's' : '';
 	$string .= join "\n", "\n\n;; ADDITIONAL SECTION ($arcount record$ars)", @additional;
 
-	return $string."\n\n";
+	return "$string\n\n";
 }
 
 
@@ -388,8 +425,8 @@ sub answersize {
 
 =head2 push
 
-    $ancount = $packet->push(pre        => $rr);
-    $nscount = $packet->push(update     => $rr);
+    $ancount = $packet->push(prereq => $rr);
+    $nscount = $packet->push(update => $rr);
     $arcount = $packet->push(additional => $rr);
 
     $nscount = $packet->push(update => $rr1, $rr2, $rr3);
@@ -406,11 +443,10 @@ sub push {
 	my $section = lc shift || '';
 	my @rr	    = grep ref($_), @_;
 
-	my $hdr = $self->{header};
 	for ($section) {
-		return $hdr->qdcount( CORE::push( @{$self->{question}}, @rr ) ) if /^question/;
+		return CORE::push( @{$self}{question}, @rr ) if /^question/;
 
-		if ( $hdr->opcode eq 'UPDATE' ) {
+		if ( $self->header->opcode eq 'UPDATE' ) {
 			my ($zone) = $self->zone;
 			my $zclass = $zone->zclass;
 			foreach (@rr) {
@@ -418,9 +454,9 @@ sub push {
 			}
 		}
 
-		return $hdr->ancount( CORE::push( @{$self->{answer}},	  @rr ) ) if /^ans|^pre/;
-		return $hdr->nscount( CORE::push( @{$self->{authority}},  @rr ) ) if /^auth|^upd/;
-		return $hdr->adcount( CORE::push( @{$self->{additional}}, @rr ) ) if /^add/;
+		return CORE::push( @{$self}{answer},	 @rr ) if /^ans|^pre/;
+		return CORE::push( @{$self}{authority},	 @rr ) if /^auth|^upd/;
+		return CORE::push( @{$self}{additional}, @rr ) if /^add/;
 	}
 
 	carp qq(invalid section "$section");
@@ -430,8 +466,8 @@ sub push {
 
 =head2 unique_push
 
-    $ancount = $packet->unique_push(pre        => $rr);
-    $nscount = $packet->unique_push(update     => $rr);
+    $ancount = $packet->unique_push(prereq => $rr);
+    $nscount = $packet->unique_push(update => $rr);
     $arcount = $packet->unique_push(additional => $rr);
 
     $nscount = $packet->unique_push(update => $rr1, $rr2, $rr3);
@@ -476,13 +512,10 @@ sub pop {
 	my $section = lc shift || '';
 
 	for ($section) {
-		return CORE::pop( @{$self->{question}} ) if /^question/;
-
-		$self->additional if $self->{buffer};		# decode remaining data
-
-		return CORE::pop( @{$self->{answer}} )	   if /^ans|^pre/;
-		return CORE::pop( @{$self->{authority}} )  if /^auth|^upd/;
-		return CORE::pop( @{$self->{additional}} ) if /^add/;
+		return CORE::pop( @{$self}{additional} ) if /^add/;
+		return CORE::pop( @{$self}{answer} )	 if /^ans|^pre/;
+		return CORE::pop( @{$self}{authority} )	 if /^auth|^upd/;
+		return CORE::pop( @{$self}{question} )	 if /^question/;
 	}
 
 	carp qq(invalid section "$section");
@@ -490,6 +523,7 @@ sub pop {
 }
 
 
+
 =head2 dn_comp
 
     $compname = $packet->dn_comp("foo.example.com", $offset);
@@ -600,7 +634,7 @@ sub dn_expand_PP {
     $response = $res->send($update);
 
 Attaches a TSIG resource record object containing a key, which will
-be used to signs a packet with a TSIG resource record (see RFC 2845).
+be used to sign a packet with a TSIG resource record (see RFC 2845).
 Uses the following defaults:
 
     algorithm   = HMAC-MD5.SIG-ALG.REG.INT
@@ -634,7 +668,12 @@ sub sign_tsig {
 		$tsig = Net::DNS::RR->new("$tsig TSIG $key");
 	}
 
-	$self->push('additional', $tsig) if $tsig;
+	my @additional = grep { $_->type ne 'TSIG' } @{$self->{additional}};
+	my $arcount = scalar @{$self->{additional}};
+	carp("A TSIG RR is already present, removing...") if scalar @additional < $arcount;
+
+	CORE::push( @additional, $tsig ) if $tsig;
+	@{$self}{additional} = [@additional];
 	return $tsig;
 }
 
@@ -768,6 +807,15 @@ sub truncate {
 }
 
 
+sub dump {				## print internal data structure
+	use Data::Dumper;
+	$Data::Dumper::Sortkeys = sub { return [sort keys %{$_[0]}] };
+	my $self = shift;
+	return Dumper($self) if defined wantarray;
+	print Dumper($self);
+}
+
+
 1;
 __END__
 
