@@ -1,6 +1,6 @@
 package Net::DNS::Resolver::Base;
 #
-# $Id: Base.pm 979 2012-01-27 08:43:38Z willem $
+# $Id: Base.pm 1005 2012-08-23 13:44:16Z willem $
 #
 
 use strict;
@@ -24,7 +24,7 @@ use IO::Select;
 use Net::DNS;
 use Net::DNS::Packet;
 
-$VERSION = (qw$LastChangedRevision: 979 $)[1];
+$VERSION = (qw$LastChangedRevision: 1005 $)[1];
 
 
 #
@@ -313,6 +313,12 @@ sub searchlist {
 	return @{$self->{'searchlist'}};
 }
 
+sub empty_searchlist {
+	my $self = shift;
+	$self->{'searchlist'} = [];
+	return $self->searchlist();
+}
+
 sub nameservers {
     my $self   = shift;
 
@@ -377,6 +383,12 @@ sub nameservers {
     return @returnval;
 }
 
+sub empty_nameservers {
+	my $self = shift;
+	$self->{'nameservers'} = [];
+	return $self->nameservers();
+}
+
 sub nameserver { &nameservers }
 
 sub cname_addr {
@@ -606,9 +618,10 @@ sub send_tcp {
 				next;
 			}
 
-			my ($ans, $err) = Net::DNS::Packet->new(\$buf, $self->{'debug'});
+			my $ans = Net::DNS::Packet->new(\$buf, $self->{debug});
+			$self->errorstring($@);
+
 			if (defined $ans) {
-				$self->errorstring($ans->header->rcode);
 				$ans->answerfrom($self->answerfrom);
 
 				if ($ans->header->rcode ne "NOERROR" &&
@@ -621,10 +634,6 @@ sub send_tcp {
 				}
 
 			}
-			elsif (defined $err) {
-				$self->errorstring($err);
-			}
-
 			return $ans;
 		}
 		else {
@@ -872,12 +881,12 @@ sub send_udp {
 				  length($buf), " bytes\n"
 				      if $self->{'debug'};
 
-				  my ($ans, $err) = Net::DNS::Packet->new(\$buf, $self->{'debug'});
+				  my $ans = Net::DNS::Packet->new(\$buf, $self->{debug});
+				  $self->errorstring($@);
 
 				  if (defined $ans) {
 				      next SELECTOR unless ( $ans->header->qr || $self->{'ignqrid'});
 				      next SELECTOR unless  ( ($ans->header->id == $packet->header->id) || $self->{'ignqrid'} );
-				      $self->errorstring($ans->header->rcode);
 				      $ans->answerfrom($self->answerfrom);
 				      if ($ans->header->rcode ne "NOERROR" &&
 					  $ans->header->rcode ne "NXDOMAIN"){
@@ -890,10 +899,9 @@ sub send_udp {
 					  next NAMESERVER ;
 
 				      }
-				  } elsif (defined $err) {
-				      $self->errorstring($err);
 				  }
 				  return $ans;
+
 			      } else {
 				  $self->errorstring($!);
       				  print ';; recv ERROR(',
@@ -1056,16 +1064,12 @@ sub bgread {
 		      $sock->peerport, ' : ', length($buf), " bytes\n"
 			if $self->{'debug'};
 
-		my ($ans, $err) = Net::DNS::Packet->new(\$buf, $self->{'debug'});
-
-		if (defined $ans) {
-			$self->errorstring($ans->header->rcode);
-			$ans->answerfrom($sock->peerhost);
-		} elsif (defined $err) {
-			$self->errorstring($err);
-		}
+		my $ans = Net::DNS::Packet->new(\$buf, $self->{debug});
+		$self->errorstring($@);
 
+		$ans->answerfrom($sock->peerhost) if defined $ans;
 		return $ans;
+
 	} else {
 		$self->errorstring($!);
 		return;
@@ -1302,8 +1306,8 @@ sub axfr_next {
 			return wantarray ? (undef, $err) : undef;
 		}
 
-		my $ans;
-		($ans, $err) = Net::DNS::Packet->new(\$buf, $self->{'debug'});
+		my $ans = Net::DNS::Packet->new(\$buf, $self->{debug});
+		my $err = $@;
 
 		if ($ans) {
 			if ($ans->header->rcode ne 'NOERROR') {
@@ -1338,7 +1342,7 @@ sub axfr_next {
 
 		if ($self->{'axfr_soa_count'} >= 2) {
 			$self->{'axfr_sel'} = undef;
-			# we need to mark the transfer as over if the responce was in
+			# we need to mark the transfer as over if the response was in
 			# many answers.  Otherwise, the user will call axfr_next again
 			# and that will cause a 'no transfer in progress' error.
 			push(@{$self->{'axfr_rr'}}, undef);
